package com.ruoyi.web.runner;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.io.file.FileReader;
import com.alibaba.druid.pool.DruidDataSource;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.TypeReference;
import com.ruoyi.framework.datasource.DynamicDataSource;
import com.ruoyi.quartz.glue.GlueFactory;
import com.ruoyi.quartz.handler.IJobHandler;
import com.ruoyi.quartz.handler.impl.MyJobHandler;
import com.ruoyi.ucp.entity.Formula;
import com.ruoyi.ucp.entity.Method;
import com.ruoyi.ucp.entity.ProBasicModelPowerRd;
import com.ruoyi.ucp.entity.UcpDataSource;
import com.ruoyi.ucp.feign.AdapterApi;
import com.ruoyi.ucp.service.*;
import com.ruoyi.ucp.util.CalcCache;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.CommandLineRunner;
import org.springframework.data.redis.connection.lettuce.LettuceConnectionFactory;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.io.File;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Component
public class LoadDataSourceRunner implements CommandLineRunner {
    @Resource
    private DynamicDataSource dynamicDataSource;
    @Resource
    private IUcpDataSourceService ucpDataSourceService;

    //@Resource
    //private ScriptEngine engine;
    @Resource
    private AdapterApi adapter;
    @Resource
    private IPointInfoService pointService;
    @Resource
    private IStationInfoDayService stationInfoDayService;
    @Resource
    private IStationInfoHourService stationInfoHourService;
    @Resource
    private IMethodService methodService;
    @Resource
    private ILineInfoDayService lineInfoDayService;
    @Resource
    private IFormulaService formulaService;
    @Resource
    @Autowired
    private RedisTemplate redisTemplate;
    @Value("${ruoyi.profile}")
    private String path;

    @Override
    public void run(String... args) {
        List<UcpDataSource> list = ucpDataSourceService.list();
        List<DruidDataSource> dataSources = BeanUtil.copyToList(list, DruidDataSource.class);
        dynamicDataSource.createDataSource(dataSources);

        //engine.put("adapter", adapter);
        //engine.put("pointService", pointService);
        //engine.put("stationInfoDayService", stationInfoDayService);
        //engine.put("stationInfoHourService", stationInfoHourService);
        //engine.put("lineInfoDayService", lineInfoDayService);

        List<Method> methods = methodService.list();
        List<Formula> formulas = formulaService.list();
        //        String functionCode = GlueFactory.getFunctionTemplate();
        String jarPath = path + "FunctionTemplate.java";
        String functionCode = FileReader.create(new File(jarPath)).readString();
        //        String functionCode = ResourceUtil.readUtf8Str("FunctionTemplate.java");
        try {
            for (Method method : methods) {
                //engine.eval(method.getMethodExpression());
                functionCode = functionCode
                        .replace("//此处加公式", method.getMethodExpression() + "\n//此处加公式");
            }
            for (Formula formula : formulas) {
                functionCode = functionCode.replace("//此处加公式", formula.getFormulaExpression() + "\n//此处加公式");
            }
            IJobHandler jobHandler = GlueFactory.getInstance().loadNewInstance(functionCode);
            MyJobHandler.jobHandler = jobHandler;
        } catch (Exception e) {
            e.printStackTrace();
        }

        LettuceConnectionFactory factory = (LettuceConnectionFactory) redisTemplate.getConnectionFactory();
        int database = factory.getDatabase();
        factory.setDatabase(9);
        redisTemplate.setConnectionFactory(factory);
        factory.afterPropertiesSet();
        factory.resetConnection();
        /*Set<String> keys = redisTemplate.keys("monthCurve_*");
        for (String key : keys) {
            //redis缓存中获取到字符串
            JSONArray o = (JSONArray) redisTemplate.opsForValue().get(key);
            //字符串类型转换为list月拟合曲线集合
            List<ProEconWtCurveFittingMonth> powerList = JSON.parseObject(o.toJSONString(), new TypeReference<List<ProEconWtCurveFittingMonth>>() {}.getType());

//            List<ProEconWtCurveFittingMonth> powerList = JSONObject.parseObject((String) o, new TypeReference<List<ProEconWtCurveFittingMonth>>() {});
            Map<Double, Double> collect = powerList.stream().collect(Collectors.toMap(ProEconWtCurveFittingMonth::getSpeed, ProEconWtCurveFittingMonth::getActualPower,(v1,v2)->v1));
            CalcCache.wtMcfMap.put(powerList.get(0).getWindturbineId(), collect);
        }*/

        String zllgl = (String) redisTemplate.opsForValue().get("ZLLGL");
        //字符串类型转换为list月拟合曲线集合
        Map<String, Map<Double, ProBasicModelPowerRd>> bzgl = JSON.parseObject(zllgl, new TypeReference<Map<String, Map<Double, ProBasicModelPowerRd>>>() {
        }.getType());
        Map<String, Map<Double, Double>> bz = new HashMap<>();
        bzgl.forEach((modeId, map) -> {
            Map<Double, Double> mdd = new HashMap<>();
            map.forEach((speed, mprd) -> {
                mdd.put(speed, mprd.getEnsurePower());
            });
            bz.put(modeId, mdd);
        });
        CalcCache.bzgl = bz;

        factory.setDatabase(database);
        factory.afterPropertiesSet();
        factory.resetConnection();
        String glqxnh = (String) redisTemplate.opsForValue().get("glqxnh");
        CalcCache.fitcoef = JSON.parseObject(glqxnh, new TypeReference<Map<String, double[]>>() {
        }.getType());
    }
}
